<?php
use CRM_Documents_ExtensionUtil as E;

class CRM_Documents_BAO_Document extends CRM_Documents_DAO_Document {

  /**
   * Create or update a record from supplied params.
   *
   * If 'id' is supplied, an existing record will be updated
   * Otherwise a new record will be created.
   *
   * @param array $record
   *
   * @return static
   * @throws \CRM_Core_Exception
   */
  public static function writeRecord(array $record): CRM_Documents_DAO_Document {
    $idField = static::$_primaryKey[0];
    $op = empty($record[$idField]) ? 'create' : 'edit';
    if ($op == 'create') {
      if (empty($record['date_added'])) {
        $record['date_added'] = date('Ymd');
      }
      if (empty($record['added_by'])) {
        $record['added_by'] = CRM_Core_Session::getLoggedInContactID();
      }
    }
    if (empty($record['date_updated'])) {
      $record['date_updated'] = date('Ymd');
    }
    if (empty($record['updated_by'])) {
      $record['updated_by'] = CRM_Core_Session::getLoggedInContactID();
    }
    $instance = parent::writeRecord($record);
    if (!empty($record['case_id'])) {
      try {
        $documentsRepo = CRM_Documents_Entity_DocumentRepository::singleton();
        $id = $instance->id;
        $caseId = $record['case_id'];
        $document = $documentsRepo->getDocumentById($id);
        $case = civicrm_api3('Case', 'getsingle', array("case_id" => $caseId));
        $document->addCaseid($caseId);
        $document->setContactIds($case['client_id']);
        $documentsRepo->persistContacts($document);
        $documentsRepo->persistCases($document);
        $documentsRepo->flush();
      } catch (\Throwable $e) {
        // Do nothing.
      }
    }
    if (!empty($record['contact_id'])) {
      try {
        $documentsRepo = CRM_Documents_Entity_DocumentRepository::singleton();
        $id = $instance->id;
        $document = $documentsRepo->getDocumentById($id);
        if (is_array($record['contact_id'])) {
          foreach ($record['contact_id'] as $contactId) {
            $document->addContactId($contactId);
          }
        } else {
          $document->addContactId($record['contact_id']);
        }
        $documentsRepo->persistContacts($document);
        $documentsRepo->flush();
      } catch (\Throwable $e) {
        // Do nothing.
      }
    }
    return $instance;
  }
  /**
   * Create a new Document based on array-data
   *
   * @param array $params key-value pairs
   * @return CRM_Documents_DAO_Document|NULL
   *
  public static function create($params) {
    $className = 'CRM_Documents_DAO_Document';
    $entityName = 'Document';
    $hook = empty($params['id']) ? 'create' : 'edit';

    CRM_Utils_Hook::pre($hook, $entityName, CRM_Utils_Array::value('id', $params), $params);
    $instance = new $className();
    $instance->copyValues($params);
    $instance->save();
    CRM_Utils_Hook::post($hook, $entityName, $instance->id, $instance);

    return $instance;
  } */



}
