<?php
/**
 * Copyright (C) 2025  Jaap Jansma (jaap.jansma@civicoop.org)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

namespace Civi\Api4\Service\Spec\Provider;

use Civi\Api4\Service\Spec\FieldSpec;
use Civi\Api4\Service\Spec\Provider\Generic\SpecProviderInterface;
use Civi\Api4\Service\Spec\RequestSpec;
use Civi\Core\Service\AutoService;
use CRM_Documents_ExtensionUtil as E;

class DocumentVersionCreateSpecProvider extends AutoService implements SpecProviderInterface {
  /**
   * @param \Civi\Api4\Service\Spec\RequestSpec $spec
   *
   * @return void
   */
  public function modifySpec(RequestSpec $spec)
  {
    $field = new FieldSpec('fid', $spec->getEntity(), 'Integer');
    $field->setLabel(E::ts('File'))
      ->setTitle(E::ts('File'))
      ->setColumnName('id')
      ->setInputType('File')
      ->setType('Extra')
      ->setFkEntity('DocumentVersion')
      ->setReadonly(FALSE)
    ;
    $spec->addFieldSpec($field);
  }

  /**
   * @param string $entity
   * @param string $action
   *
   * @return bool
   */
  public function applies(string $entity, string $action)
  {
    return $entity == 'DocumentVersion' && in_array($action, ['create', 'save', 'update']);
  }


}