<?php
/**
 * Copyright (C) 2025  Jaap Jansma (jaap.jansma@civicoop.org)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

use CRM_Documents_ExtensionUtil as E;

return [
  [
    'name' => 'SavedSearch_Case_Documents',
    'entity' => 'SavedSearch',
    'cleanup' => 'always',
    'update' => 'unmodified',
    'params' => [
      'version' => 4,
      'values' => [
        'name' => 'Case_Documents',
        'label' => E::ts('Case Documents'),
        'form_values' => [
          'join' => [
            'DocumentCase_Document_document_id_01' => 'Document',
            'DocumentCase_Document_document_id_01_Document_DocumentContact_document_id_01' => 'DocumentContact',
          ],
        ],
        'api_entity' => 'DocumentCase',
        'api_params' => [
          'version' => 4,
          'select' => [
            'case_id',
            'DocumentCase_Document_document_id_01.id',
            'DocumentCase_Document_document_id_01.date_updated',
            'DocumentCase_Document_document_id_01.date_added',
            'DocumentCase_Document_document_id_01.subject',
            'DocumentCase_Document_document_id_01.status_id:label',
            'DocumentCase_Document_document_id_01.type_id:label',
            'DocumentCase_Document_document_id_01.added_by.display_name',
            'DocumentCase_Document_document_id_01.updated_by.display_name',
            'GROUP_CONCAT(DocumentCase_Document_document_id_01_Document_DocumentContact_document_id_01.contact_id.display_name ORDER BY DocumentCase_Document_document_id_01_Document_DocumentContact_document_id_01.contact_id.display_name ASC) AS GROUP_CONCAT_DocumentCase_Document_document_id_01_Document_DocumentContact_document_id_01_contact_id_display_name_DocumentCase_Document_document_id_01_Document_DocumentContact_document_id_01_contact_id_display_name',
            'DocumentCase_Document_document_id_01.download_url',
          ],
          'orderBy' => [],
          'where' => [],
          'groupBy' => [
            'DocumentCase_Document_document_id_01.id',
          ],
          'join' => [
            [
              'Document AS DocumentCase_Document_document_id_01',
              'INNER',
              [
                'document_id',
                '=',
                'DocumentCase_Document_document_id_01.id',
              ],
            ],
            [
              'DocumentContact AS DocumentCase_Document_document_id_01_Document_DocumentContact_document_id_01',
              'LEFT',
              [
                'DocumentCase_Document_document_id_01.id',
                '=',
                'DocumentCase_Document_document_id_01_Document_DocumentContact_document_id_01.document_id',
              ],
            ],
          ],
          'having' => [],
        ],
      ],
      'match' => [
        'name',
      ],
    ],
  ],
  [
    'name' => 'SavedSearch_Case_Documents_SearchDisplay_Case_Documents_Tabel',
    'entity' => 'SearchDisplay',
    'cleanup' => 'always',
    'update' => 'unmodified',
    'params' => [
      'version' => 4,
      'values' => [
        'name' => 'Case_Documents_Tabel',
        'label' => E::ts('Case Documents Tabel'),
        'saved_search_id.name' => 'Case_Documents',
        'type' => 'table',
        'settings' => [
          'description' => NULL,
          'sort' => [
            [
              'DocumentCase_Document_document_id_01.date_updated',
              'DESC',
            ],
          ],
          'limit' => 50,
          'pager' => [],
          'placeholder' => 5,
          'columns' => [
            [
              'type' => 'field',
              'key' => 'DocumentCase_Document_document_id_01.subject',
              'dataType' => 'String',
              'label' => E::ts('Subject'),
              'sortable' => TRUE,
            ],
            [
              'type' => 'field',
              'key' => 'DocumentCase_Document_document_id_01_Document_DocumentContact_document_id_01.contact_id.display_name',
              'dataType' => 'String',
              'label' => E::ts('Contacts'),
              'sortable' => TRUE,
            ],
            [
              'type' => 'field',
              'key' => 'DocumentCase_Document_document_id_01.type_id:label',
              'dataType' => 'Integer',
              'label' => E::ts('Type'),
              'sortable' => TRUE,
            ],
            [
              'type' => 'field',
              'key' => 'DocumentCase_Document_document_id_01.status_id:label',
              'dataType' => 'Integer',
              'label' => E::ts('Status'),
              'sortable' => TRUE,
            ],
            [
              'type' => 'field',
              'key' => 'DocumentCase_Document_document_id_01.date_updated',
              'dataType' => 'Timestamp',
              'label' => E::ts('Date Modified'),
              'sortable' => TRUE,
            ],
            [
              'type' => 'field',
              'key' => 'DocumentCase_Document_document_id_01.updated_by.display_name',
              'dataType' => 'String',
              'label' => E::ts('Modified by'),
              'sortable' => TRUE,
            ],
            [
              'links' => [
                [
                  'path' => '',
                  'icon' => 'fa-download',
                  'text' => E::ts('Download'),
                  'style' => 'default',
                  'condition' => [],
                  'task' => '',
                  'entity' => 'Document',
                  'action' => 'download',
                  'join' => 'DocumentCase_Document_document_id_01',
                  'target' => '',
                ],
              ],
              'type' => 'links',
              'alignment' => 'text-right',
              'label' => '',
            ],
            [
              'text' => '',
              'style' => 'default',
              'size' => 'btn-xs',
              'icon' => 'fa-bars',
              'links' => [
                [
                  'entity' => 'Document',
                  'action' => 'new_version',
                  'join' => '',
                  'target' => 'crm-popup',
                  'icon' => 'fa-upload',
                  'text' => E::ts('Upload new version'),
                  'style' => 'default',
                  'path' => '',
                  'task' => '',
                  'condition' => [],
                ],
                [
                  'entity' => 'Document',
                  'action' => 'edit',
                  'join' => '',
                  'target' => 'crm-popup',
                  'icon' => 'fa-pencil',
                  'text' => E::ts('Edit Document'),
                  'style' => 'default',
                  'path' => '',
                  'task' => '',
                  'condition' => [],
                ],
                [
                  'entity' => 'Document',
                  'action' => 'view_versions',
                  'join' => '',
                  'target' => 'crm-popup',
                  'icon' => 'fa-list-ol',
                  'text' => E::ts('View versions'),
                  'style' => 'default',
                  'path' => '',
                  'task' => '',
                  'condition' => [],
                ],
                [
                  'entity' => 'Document',
                  'action' => 'delete',
                  'join' => '',
                  'target' => 'crm-popup',
                  'icon' => 'fa-trash',
                  'text' => E::ts('Delete Document'),
                  'style' => 'danger',
                  'path' => '',
                  'task' => '',
                  'condition' => [],
                ],
              ],
              'type' => 'menu',
              'alignment' => 'text-right',
            ],
          ],
          'actions' => FALSE,
          'classes' => [
            'table',
            'table-striped',
          ],
          'toolbar' => [
            [
              'path' => 'civicrm/documents/document?action=add&case_id=[case_id]&context=case&reset=1',
              'icon' => 'fa-file-circle-plus',
              'text' => E::ts('New Document'),
              'style' => 'success',
              'condition' => [],
              'task' => '',
              'entity' => '',
              'action' => '',
              'join' => '',
              'target' => 'crm-popup',
            ],
          ],
          'headerCount' => TRUE,
        ],
      ],
      'match' => [
        'saved_search_id',
        'name',
      ],
    ],
  ],
];